@@ -3,7 +3,7 @@
 # IDEA: Use the flyweight pattern to limit memory usage if identical converters would
 # be created.
 
-import act # type: ignore
+import act  # type: ignore
 import logging
 import xarray as xr
 import pandas as pd
@@ -13,14 +13,21 @@ from numpy.typing import NDArray
 from pydantic import validator
 
 from ..config.dataset import DatasetConfig
-from ..utils import assign_data
-from .base import DataConverter
+from .base import DataConverter, RetrievedDataset
 
-__all__ = ["UnitsConverter", "StringToDatetime"]
+__all__ = [
+    "UnitsConverter",
+    "StringToDatetime",
+    "NearestNeighbor",
+]
 
 logger = logging.getLogger(__name__)
 
 
+# IDEA: "@data_converter()" decorator so DataConverters can be defined as functions in
+# user code. Arguments to data_converter can be parameters to the class.
+
+
 class UnitsConverter(DataConverter):
     """---------------------------------------------------------------------------------
     Converts the units of a retrieved variable to specified output units.
@@ -42,12 +49,13 @@ class UnitsConverter(DataConverter):
 
     def convert(
         self,
-        dataset: xr.Dataset,
-        dataset_config: DatasetConfig,
+        data: xr.DataArray,
         variable_name: str,
+        dataset_config: DatasetConfig,
+        retrieved_dataset: RetrievedDataset,
         **kwargs: Any,
-    ) -> xr.Dataset:
-        input_units = self._get_input_units(dataset, variable_name)
+    ) -> Optional[xr.DataArray]:
+        input_units = self._get_input_units(data)
         if not input_units:
             logger.warning(
                 "Input units for variable '%s' could not be found. Please ensure these"
@@ -56,7 +64,7 @@ class UnitsConverter(DataConverter):
                 variable_name,
                 variable_name,
             )
-            return dataset
+            return None
 
         output_units = dataset_config[variable_name].attrs.units
         if (
@@ -71,29 +79,29 @@ class UnitsConverter(DataConverter):
                     " are set in the dataset configuration file for the specified variable.",
                     variable_name,
                 )
-            return dataset
+            return None
 
-        data: NDArray[Any] = act.utils.data_utils.convert_units(  # type: ignore
-            data=dataset[variable_name].data,
+        converted: NDArray[Any] = act.utils.data_utils.convert_units(  # type: ignore
+            data=data.data,
             in_units=input_units,
             out_units=output_units,
         )
-        dataset = assign_data(dataset, data, variable_name)
-        dataset[variable_name].attrs["units"] = output_units
+        data_array = data.copy(data=converted)
+        data_array.attrs["units"] = output_units
         logger.debug(
             "Converted '%s's units from '%s' to '%s'",
             variable_name,
             input_units,
             output_units,
         )
-        return dataset
+        return data_array
 
-    def _get_input_units(self, dataset: xr.Dataset, variable_name: str) -> str:
+    def _get_input_units(self, data: xr.DataArray) -> str:
         units = ""
         if self.input_units:
             units = self.input_units
-        elif "units" in dataset[variable_name].attrs:
-            units = dataset[variable_name].attrs["units"]
+        elif "units" in data.attrs:
+            units = data.attrs["units"]
         return units
 
 
@@ -146,13 +154,13 @@ class StringToDatetime(DataConverter):
 
     def convert(
         self,
-        dataset: xr.Dataset,
-        dataset_config: DatasetConfig,
+        data: xr.DataArray,
         variable_name: str,
+        dataset_config: DatasetConfig,
+        retrieved_dataset: RetrievedDataset,
         **kwargs: Any,
-    ) -> xr.Dataset:
-        dt: NDArray[Any] = dataset[variable_name].data
-        dt = pd.to_datetime(dt, format=self.format, **self.to_datetime_kwargs)  # type: ignore
+    ) -> Optional[xr.DataArray]:
+        dt = pd.to_datetime(data.data, format=self.format, **self.to_datetime_kwargs)  # type: ignore
 
         if self.timezone and self.timezone != "UTC":
             dt = dt.tz_localize(self.timezone).tz_convert("UTC")  # type: ignore
@@ -161,8 +169,55 @@ class StringToDatetime(DataConverter):
             dt = dt.tz_localize(None)  # type: ignore
 
         dtype = dataset_config[variable_name].dtype
-        data: NDArray[Any] = np.array(dt, dtype=dtype)  # type: ignore
+        converted: NDArray[Any] = np.array(dt, dtype=dtype)  # type: ignore
 
-        dataset = assign_data(dataset, data, variable_name)
+        if variable_name in dataset_config.coords:
+            data_array = xr.DataArray(
+                data=converted,
+                dims=data.dims,
+                coords={variable_name: converted},
+                attrs=data.attrs,
+                name=variable_name,
+            )
+        else:
+            data_array = xr.DataArray(
+                data=converted,
+                dims=data.dims,
+                coords=data.coords,
+                attrs=data.attrs,
+                name=variable_name,
+            )
+        return data_array
 
-        return dataset
+
+class NearestNeighbor(DataConverter):
+    """Maps data onto the specified coordinate grid using nearest-neighbor."""
+
+    coord: str = "time"
+    """The coordinate axis this converter should be applied on. Defaults to 'time'."""
+
+    def convert(
+        self,
+        data: xr.DataArray,
+        variable_name: str,
+        dataset_config: DatasetConfig,
+        retrieved_dataset: RetrievedDataset,
+        **kwargs: Any,
+    ) -> Optional[xr.DataArray]:
+        # Assume that the coord index in the output matches coord index in the retrieved
+        # structure.
+        target_coord = retrieved_dataset.coords[self.coord]
+        coord_index = dataset_config[variable_name].dims.index(self.coord)
+        current_coord_name = tuple(data.coords.keys())[coord_index]
+
+        # Create an empty DataArray with the shape we want to achieve
+        new_coords = {
+            k: v.data if k != current_coord_name else target_coord.data
+            for k, v in data.coords.items()
+        }
+        tmp_data = xr.DataArray(coords=new_coords, dims=tuple(new_coords))
+
+        # Resample the data using nearest neighbor
+        new_data = data.reindex_like(other=tmp_data, method="nearest")
+
+        return new_data
